/*
 * Copyright (C) 2009 - 2011 Nokia Corporation.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/.
 *
 */

#ifndef PACKAGEMANAGER_H
#define PACKAGEMANAGER_H

#include <QtDBus/QDBusPendingReply>
#include <QObject>
#include <QMap>
#include <QList>
#include <QVariant>

class QString;
class QDBusError;
class QDBusPendingCallWatcher;
class PackageManagerPendingCallWatcher;
class PackageManagerPendingFetchWatcher;
class PackageManagerPrivate;

const QString PackageNameString("Name");
const QString PackageVersionString("Version");
const QString PackageInstalledSizeInt64("InstalledSize");
const QString PackageDisplayNameString("DisplayName");
const QString PackageDescriptionString("Description");
const QString PackageCategoryString("Category");
const QString PackageRepositoryString("Repository");
const QString PackageIconDataString("IconData");
const QString PackageState("State");

/*!
 * @class PackageManager
 * @brief Package Manager Qt bindings library public API header
 * @details PackageManager interface class provides Qt style wrapping for D-Bus methods 
 * exposed by Package Manager application. Any application calling a wrapped D-Bus method 
 * should connect to a dbusError signal of the PackageManagerPendingCallWatcher transaction
 * pointer returned. All provided methods are asynchronous. Any D-Bus call with a return
 * value are wrapped to a separate waiting thread that generates a signal when reply value
 * is returned.
 *
 * Package Manager application contains three separate binaries with D-Bus interfaces.
 * This class wraps all of them to allow seamless usage of whatever functionality is
 * desired. The interfaces consists of:
 * - com.nokia.package_manager: a restricted interface to a middleware application
 *   (no UI) that directly uses platform packaging tools.
 * - com.nokia.package_manager_install_ui: a simple MeeGo touch application responsible of
 *   showing any confirmation and error dialogs during package operations.
 * - com.nokia.package_manager_ui: interface provided by Package Manager UI application to
 *   launch it directly to a specified UI page or to show a certain package on
 *   installations page.
 *
 * Methods provided by this class can roughly be divided into four categories:
 * - Data fetches. Simple data querying methods prefixed with "fetch". Each fetch call
 *   returns a pointer to a PackageManagerPendingFetchWatcher object that wraps the
 *   transaction. A reply signal is generated by the library ONLY to the application that
 *   calls the method. The reply signal is always "dataFetched" from
 *   PackageManagerPendingFetchWatcher, but an overloaded version depends on called query.
 *   Failed fetch is indicated by a dbusError signal, even if the actually D-Bus call is
 *   handled correctly.
 * - Package operations provided by com.nokia.package_manager. Only one package operation
 *   can be active at any time (though it does not prevent data fetches). Any operation
 *   progress and completion signals will be broadcasted as D-Bus signals to all connected
 *   applications. Due to security reasons, these methods are D-Bus protected (any 3rd
 *   party developers should use the similar methods provided by
 *   com.nokia.package_manager_install_ui).
 * - Package operations provided by com.nokia.package_manager_install_ui. Invokes Package
 *   Manager Installation UI that takes care of showing any confirmation and error
 *   dialog. Applications calling these methods may still listen to progress signals if
 *   they choose so.
 * - Methods to open a specified page of Package Manager UI Application. For example, an
 *   application may choose to display currently available package updates, or to display
 *   an installation progress of a specified package on an installations page.
 */

class PackageManager : public QObject
{
    Q_OBJECT

public:
    /*!
     * @brief This enum contains all possible error codes used by Package Manager
     */
    enum ErrorCode
    {
        //! No error
        ErrorNone = 0,
        //! D-Bus error (such as no connection to interface)
        ErrorDbus,
        //! Unrecoverable error caused by platform tools
        ErrorInternal,
        //! Invalid argument was passed to a method
        ErrorInvalidArgument,
        //! No network connection to refresh repository cache
        ErrorCheckUpdatesFailed,
        //! Specified file does not exist
        ErrorFileNotFound,
        //! Specified file is not in supported format
        ErrorInvalidFile,
        //! Specified package is not user package
        ErrorPackageNotUser,
        //! Package exists in cache but was not found on repository. It might have been removed.
        ErrorNoCandidate,
        //! There is no package with the specified name
        ErrorPackageNotFound,
        //! Specified version of the package does not exist
        ErrorPackageVersionNotFound,
        //! There is no package with the specified name installed
        ErrorPackageNotInstalled,
        //! Package is incompatible with the device
        ErrorPackageNotCompatible,
        //! Some other package depends on the package
        ErrorDependenciesPreventUninstall,
        //! Some of the packages needed by the specified package could not be found
        ErrorMissingDependencies,
        //! Download of one or more packages failed
        ErrorDownloadFailed,
        //! Operation started but failed before finishing
        ErrorOperationFailed,
        //! Upgrade operation started but failed before finishing
        ErrorUpgradeFailed,
        //! Specified package is already installed
        ErrorPackageAlreadyInstalled,
        //! Newest version of the specified package is already installed
        ErrorPackageAlreadyNewest,
        //! Trying to install older version of a package that is currently installed
        ErrorNewerVersionInstalled,
        //! Trying to install a package that requires another package to be downgraded
        ErrorRequiresOlderVersionOfDependency,
        //! Invalid operation requested
        ErrorInvalidOperation,
        //! No ongoing operations
        ErrorNoOperation,
        //! Operation is not in a cancelable state
        ErrorNotCancelable,
        //! Operation was canceled
        ErrorOperationCanceled,
        //! An unspecified error occured
        ErrorUnknown
    };

    /*!
     * @brief Constructs Package Manager D-Bus interface wrapper
     */
    PackageManager();

    /*!
     * @brief Destructs the interface wrapper
     *
     * @note Destroying this class has no effect in ongoing package operations.
     */
    ~PackageManager();

    /*!
     * @brief A D-Bus call timeout in milliseconds. Default is 24h.
     */
    int callTimeout;

public Q_SLOTS: // METHODS from com.nokia.package_manager, unrestricted

    /*!
     * @brief Fetch current operation (install, upgrade...) data.
     * @return Pointer to watcher class, ownership not returned.
     * @details The result is a key-value map. If no operation is in progress, the map is 
     * empty.
     *
     * During an operation the following keys are always present:
     *
     * - Operation: {Install, InstallFile, Upgrade, UpgradeAll, Uninstall,
     *   DownloadSystemUpgrade, SystemUpgrade, Refresh, Restore}
     *  
     * The following keys are present when Operation is one of
     * {Install, Upgrade, Uninstall, DownloadSystemUpgrade, SystemUpgrade, Refresh,
     * Restore}:
     *
     * - DownloadCurrentBytes (int64): Already downloaded bytes (-1
     *   before download is started, 0..DownloadTotalBytes during
     *   download)
     * - DownloadTotalBytes (int64): Total bytes to download (-1 before
     *   download is started)
     *
     * The following keys are present when Operation is one of
     * {Install, InstallFile, Upgrade, Uninstall, DownloadSystemUpgrade, SystemUpgrade,
     * Restore}:
     *
     * - Name: Target package name
     * - Version: Target package version
     * - OperationProgressPct (int): -1 before actual package
     *   operation, 0..100 during operation.
     * 
     * When Operation is UpgradeAll, the following keys may be present:
     * 
     * - PendingPackages: String list of packages yet to be upgraded in the operation.
     *   Each string is in format "Name=Version".
     * 
     * - RejectedPackages: List of rejected packages. These packages have a update
     *   available, but are not updated. Each entry in the list is a string of the form
     *   "Name=Version=Error".
     * 
     *  Required Aegis token: none.
     */
    PackageManagerPendingFetchWatcher* fetchCurrentOperationData();

    /*!
     * @brief Queries for available packages that have not yet been installed.
     * @return Pointer to watcher class, ownership not returned.
     * @param category Optional category filter.
     * @details If query fails due to a D-Bus error, a dbusError signal is generated by
     * PackageManagerPendingFetchWatcher. If query is handled by the middleware, fetched
     * data is returned with dataFetched(const QList< QMap<QString, QVariant> >&)
     * signal.
     *
     * Ownership of PackageManagerPendingFetchWatcher pointer is not returned.
     *
     * Required Aegis token: none.
     */
    PackageManagerPendingFetchWatcher* fetchPackages(const QString &category = "");

    /*!
     * @brief Queries for updates available to installed packages.
     * @param category Optional category filter.
     * @param refresh Optionally clients may choose to refresh package cache.
     * @return Pointer to watcher class, ownership not returned.
     * @details If query fails due to a D-Bus error, a dbusError signal is generated by
     * PackageManagerPendingFetchWatcher. If query is handled by the middleware, fetched
     * data is returned with dataFetched(const QList< QMap<QString, QVariant> >&) 
     * signal.
     * 
     * Ownership of PackageManagerPendingFetchWatcher pointer is not returned.
     * 
     * Required Aegis token: none.
     */
    PackageManagerPendingFetchWatcher* fetchAvailableUpdates(const QString &category = "");

    /*!
     * @brief Queries for installed packages.
     * @return Pointer to watcher class, ownership not returned.
     * @details If query fails due to a D-Bus error, dbusError signal is generated by
     * PackageManagerPendingFetchWatcher. If query is handled by the middleware, fetched
     * data is returned with dataFetched(const QList< QMap<QString, QVariant> >&) 
     * signal.
     * 
     * Ownership of PackageManagerPendingFetchWatcher pointer is not returned.
     * 
     * Required Aegis token: none.
     */
    PackageManagerPendingFetchWatcher* fetchInstalled();

    /*!
     * @brief Queries for meta-data of a local package file
     * @param filename Full path and name of the file.
     * @return Pointer to watcher class, ownership not returned.
     * @details If query fails due to a D-Bus error, dbusError signal is generated by
     * PackageManagerPendingFetchWatcher. If query is handled by the middleware, fetched
     * data is returned with dataFetched(const QMap<QString, QVariant>& package) signal.
     * 
     * Ownership of PackageManagerPendingFetchWatcher pointer is not returned.
     * 
     * Required Aegis token: none.
     */
    PackageManagerPendingFetchWatcher* fetchPackageDataFile(const QString &filename);

    /*!
     * @brief Queries for a meta-data of a package from one of configured repositories
     * @param name Package name. Case-sensitive.
     * @param version Optional version specifier.
     * @return Pointer to watcher class, ownership not returned.
     * @details If query fails due to a D-Bus error, dbusError signal is generated by
     * PackageManagerPendingFetchWatcher. If query is handled by the middleware, fetched
     * data is returned with dataFetched(const QMap<QString, QVariant>& package) signal.
     * 
     * Ownership of PackageManagerPendingFetchWatcher pointer is not returned.
     * 
     * Required Aegis token: none.
     */
    PackageManagerPendingFetchWatcher* fetchPackageDataRemote(const QString &packagename, const QString& version = QString());

    /*!
     * @brief Queries for meta-data of an installed package
     * @param packagename Package name. Case-sensitive.
     * @return Pointer to watcher class, ownership not returned.
     * @details If query fails due to a D-Bus error, dbusError signal is generated by
     * PackageManagerPendingFetchWatcher. If query is handled by the middleware, fetched
     * data is returned with dataFetched(const QMap<QString, QVariant>& package) signal.
     * 
     * Ownership of PackageManagerPendingFetchWatcher pointer is not returned.
     * 
     * Required Aegis token: none.
     */
    PackageManagerPendingFetchWatcher* fetchPackageDataInstalled(const QString &packagename);

    /*!
     * @brief Queries for a list of files included in an installed package
     * @param packagename Package name. Case-sensitive.
     * @return Pointer to watcher class, ownership not returned.
     * @details If query fails due to a D-Bus error, dbusError signal is generated by 
     * PackageManagerPendingFetchWatcher. If query is handled by the middleware, fetched
     * data is returned with dataFetched(const QStringList& fileList) signal.
     * 
     * Ownership of PackageManagerPendingFetchWatcher pointer is not returned.
     * 
     * Required Aegis token: none.
     */
    PackageManagerPendingFetchWatcher* fetchFilesOfInstalledPackage(const QString &packagename);

    /*!
     * @brief Queries for all dependencies of a package.
     * @param packagename Package name. Case-sensitive.
     * @return Pointer to watcher class, ownership not returned.
     * @details If query fails due to a D-Bus error, dbusError signal is generated by 
     * PackageManagerPendingFetchWatcher. If query is handled by the middleware, fetched
     * data is returned with dataFetched(const QList< QMap<QString, QVariant> >&) 
     * signal.
     * 
     * Ownership of PackageManagerPendingFetchWatcher pointer is not returned.
     * 
     * Required Aegis token: none.
     */
    PackageManagerPendingFetchWatcher* fetchPackageDependencyData(const QString &packagename);
    
    /*!
     * @brief Queries for status of a restored content.
     * @return Pointer to watcher class, ownership not returned.
     * @details If query fails due to a D-Bus error, dbusError signal is generated by
     * PackageManagerPendingFetchWatcher. If query is handled by the middleware, fetched
     * data is returned with dataFetched(const QString& data) signal.
     * 
     * Ownership of PackageManagerPendingFetchWatcher pointer is not returned.
     * 
     * Required Aegis token: none.
     */
    PackageManagerPendingFetchWatcher* fetchRestoreStatus();
    
    /*!
     * @brief Queries for a package list of a restored content.
     * @return Pointer to watcher class, ownership not returned.
     * @details If query fails due to a D-Bus error, dbusError signal is generated by
     * PackageManagerPendingFetchWatcher. If query is handled by the middleware, fetched
     * data is returned with dataFetched(const QList< QMap<QString, QVariant> >& data)
     * signal.
     * 
     * Ownership of PackageManagerPendingFetchWatcher pointer is not returned.
     * 
     * Required Aegis token: none.
     */
    PackageManagerPendingFetchWatcher* fetchRestorablePackages();

    /*!
     * @brief Simulate method.
     * @return Pointer to watcher class, ownership not returned.
     * @details If method fails due to a D-Bus error, dbusError signal is generated by
     * PackageManagerPendingFetchWatcher. If query is handled by the middleware, fetched
     * data is returned with dataFetched(const QMap<QString, QVariant>& data) signal.
     * 
     * Ownership of PackageManagerPendingFetchWatcher pointer is not returned.
     * 
     * Required Aegis token: none.
     */
    PackageManagerPendingFetchWatcher* simulateOperation(const QString &operation, const QString &packageidentifier, const QString &version, const QString &flags);

public Q_SLOTS: // METHODS from com.nokia.package_manager restricted to PM UI / Install UI only
    /*!
     * @brief Adds a package in the ignored update package list.
     * @param packagename A package to be ignored.
     * @param version A version of the package to be ignored.
     * @return Pointer to watcher class, ownership not returned.
     * @details Sends an asyncronous D-Bus method call to com.nokia.package_manager. If the call
     * itself fails, a dbusError signal will be emitted by
     * PackageManagerPendingCallWatcher. If not, the addition can be assumed as
     * succeeded.
     * 
     * Ownership of PackageManagerPendingCallWatcher pointer is not returned.
     * 
     * @credential package-manager::packagemanager_limited
     */
    PackageManagerPendingCallWatcher* ignoreUpdatePackage( const QString &packagename, const QString &version );
    
    /*!
     * @brief Refreshes only a cache specified in onlyDomain
     * @param onlyDomain Domain to be refreshed
     * @param flags Currently only "clearignored" supported, which clears ignored packages list.
     * @return Pointer to watcher class, ownership not returned.
     * @details Sends an asyncronous D-Bus method call to com.nokia.package_manager. If the call
     * itself fails, a dbusError signal will be emitted by
     * PackageManagerPendingCallWatcher. If not, Package Manager will begin the
     * operation and emits operationStarted signal and finally one operationComplete
     * signal, containing information whether the operation succeeded or failed.
     * 
     * Ownership of PackageManagerPendingCallWatcher pointer is not returned.
     * 
     * @credential package-manager::packagemanager_limited
     */
    PackageManagerPendingCallWatcher* refreshPackageCache( const QString &onlyDomain, const QString &flags );

    /*!
     * @brief Installs a local package file
     * @param filename Full path and name of the file.
     * @return Pointer to watcher class, ownership not returned.
     * @details Sends an asyncronous D-Bus method call to com.nokia.package_manager. If the call
     * itself fails, a dbusError signal will be emitted by
     * PackageManagerPendingCallWatcher. If not, Package Manager will begin the
     * operation and start signalling operationProgress signals and finally one
     * operationComplete signal, containing information whether the operation succeeded
     * or failed.
     * 
     * Ownership of PackageManagerPendingCallWatcher pointer is not returned.
     * 
     * @credential package-manager::packagemanager_limited
     */
    PackageManagerPendingCallWatcher* installFile( const QString &filename );

    /*!
     * @brief Installs a package from a configured repository
     * @param packagename Package name. Case-sensitive.
     * @param version Package version (optional).
     * @return Pointer to watcher class, ownership not returned.
     * @details Sends an asyncronous D-Bus method call to com.nokia.package_manager. If the call
     * itself fails, a dbusError signal will be emitted by
     * PackageManagerPendingCallWatcher. If not, Package Manager will begin the
     * operation and emit an operationStarted signal. After that Package Manager starts
     * signalling downloadProgress and operationProgress signals and
     * finally operationComplete signal(s), containing information whether the
     * operation succeeded or failed.
     * 
     * Ownership of PackageManagerPendingCallWatcher pointer is not returned.
     * 
     * @credential package-manager::packagemanager_limited     * 
     */
    PackageManagerPendingCallWatcher* install( const QString &packagename, const QString &version = QString() );

    /*!
     * @brief Upgrades a package from a configured repository
     * @param packagename Package name. Case-sensitive.
     * @param version Package version (optional).
     * @return Pointer to watcher class, ownership not returned.
     * @details Sends an asyncronous D-Bus method call to com.nokia.package_manager. If
     * the call itself fails, a dbusError signal will be emitted by
     * PackageManagerPendingCallWatcher. If not, Package Manager will begin the
     * operation and emit an operationStarted signal. After that Package Manager starts
     * signalling downloadProgress and operationProgress signals and finally
     * operationComplete signal(s), containing information whether the operation
     * succeeded or failed.
     * 
     * Ownership of PackageManagerPendingCallWatcher pointer is not returned.
     * 
     * @credential package-manager::packagemanager_limited
     */
    PackageManagerPendingCallWatcher* upgrade( const QString &packagename, const QString &version = QString() );

    /*!
     * @brief Restores a package from a configured repository
     * @param packagename Restorable package name. Case-sensitive.
     * @param version Restorable package version.
     * @return Pointer to watcher class, ownership not returned.
     * @details Sends an asyncronous D-Bus method call to com.nokia.package_manager. If
     * the call itself fails, a dbusError signal will be emitted by
     * PackageManagerPendingCallWatcher. If not, Package Manager will begin the
     * operation and emit an operationStarted signal. After that Package Manager starts
     * signalling downloadProgress and operationProgress signals and finally
     * operationComplete signal(s), containing information whether the operation
     * succeeded or failed.
     * 
     * Ownership of PackageManagerPendingCallWatcher pointer is not returned.
     * 
     * @credential package-manager::packagemanager_limited
     */
    PackageManagerPendingCallWatcher* restore( const QString &packagename, const QString &version );

    /*!
     * @brief Removes a package from the restorable packages list.
     * @param packagename A package to be removed.
     * @return Pointer to watcher class, ownership not returned.
     * @details Sends an asyncronous D-Bus method call to com.nokia.package_manager. If 
     * the call itself fails, a dbusError signal will be emitted by
     * PackageManagerPendingCallWatcher. If not, the removal can be assumed as
     * succeeded.
     * 
     * Ownership of PackageManagerPendingCallWatcher pointer is not returned.
     * 
     * @credential package-manager::packagemanager_limited
     */
    PackageManagerPendingCallWatcher* ignoreRestorePackage( const QString &packagename );

    /*!
     * @brief Downloads a system package from a configured repository
     * @param packagename A system package name. Case-sensitive.
     * @param version A system package version (optional).
     * @return Pointer to watcher class, ownership not returned.
     * @details Sends an asyncronous D-Bus method call to com.nokia.package_manager. If the call
     * itself fails, a dbusError signal will be emitted by
     * PackageManagerPendingCallWatcher. If not, Package Manager will begin the
     * operation and emit an operationStarted signal. After that Package Manager starts
     * signalling downloadProgress signals and finally one operationComplete signal,
     * containing information whether the operation succeeded or failed.
     * 
     * Ownership of PackageManagerPendingCallWatcher pointer is not transferred.
     * 
     * @credential package-manager::packagemanager_private
     */

    PackageManagerPendingCallWatcher* downloadSystemUpgrade( const QString &packagename, const QString &version = QString() );
    /*!
     * @brief Upgrades a system package from a configured repository
     * @param packagename A system package name. Case-sensitive.
     * @param version A system package version (optional)
     * @return Pointer to watcher class, ownership not returned.
     * @details Sends an asyncronous D-Bus method call to com.nokia.package_manager. If 
     * the call itself fails, a dbusError signal will be emitted by
     * PackageManagerPendingCallWatcher. If not, Package Manager will begin the
     * operation and emit an operationStarted signal. After that Package Manager starts
     * signalling downloadProgress and operationProgress signals and finally
     * operationComplete signal(s), containing information whether the operation
     * succeeded or failed.
     * 
     * Ownership of PackageManagerPendingCallWatcher pointer is not transferred.
     * 
     * @credential package-manager::packagemanager_private
     */
    PackageManagerPendingCallWatcher* upgradeSystem( const QString &packagename, const QString &version = QString() );

    /*!
     * @brief Uninstalls an installed package.
     * @param packagename A package name. Case-sensitive.
     * @return Pointer to watcher class, ownership not returned.
     * @details Sends an asyncronous D-Bus method call to com.nokia.package_manager. If the call
     * itself fails, a dbusError signal will be emitted by
     * PackageManagerPendingCallWatcher. If not, Package Manager will begin the
     * operation and emit an operationStarted signal. After that Package Manager starts
     * signalling operationProgress signals and finally operationComplete signal(s),
     * containing information whether the operation succeeded or failed.
     * 
     * Ownership of PackageManagerPendingCallWatcher pointer is not returned.
     * 
     * @credential package-manager::packagemanager_limited
     */
    PackageManagerPendingCallWatcher* uninstall( const QString &packagename );

    /*!
     * @brief Upgrades all packages that currently have a newer version available
     * @return Pointer to watcher class, ownership not returned.
     * @details Sends an asyncronous D-Bus method call to com.nokia.package_manager. If the call
     * itself fails, a dbusError signal will be emitted by
     * PackageManagerPendingCallWatcher. If not, Package Manager will begin the
     * operation and emit an operationStarted signal. After that Package Manager starts
     * signalling downloadProgress and operationProgress signals and finally
     * operationComplete signal(s), containing information whether the operation
     * succeeded or failed.
     * 
     * Each package is upgraded separately, one at the time. I.e. package B will begin 
     * to be downloaded (and later installed) only after package A has been downloaded 
     * and installed. In case of error(s), operation is not aborted and each error will 
     * be emitted with a separate operationComplete signal.
     * 
     * Ownership of PackageManagerPendingCallWatcher pointer is not returned.
     * 
     * @credential package-manager::packagemanager_limited
     */
    PackageManagerPendingCallWatcher* upgradeAll();

    /*!
     * @brief Cancel the current operation
     * @param operation The operation to be cancelled.
     * @param packagename A package name
     * @param version A package version
     * @return Pointer to watcher class, ownership not returned.
     * @details Sends a syncronous D-Bus method call to com.nokia.package_manager. If the call
     * itself fails, a dbusError signal will be emitted by
     * PackageManagerPendingCallWatcher. If not, Package Manager will cancel the
     * current operation matching to operation, packagename and version. If there is no
     * operation going on Package Manager will cancel a failed package installation with
     * specified package and version. In that case operation must be one of the
     * following: Install, Upgrade or InstallFile. Package Manager will send an
     * operationCompleted signal if the removal was successful.
     * 
     * Ownership of PackageManagerPendingCallWatcher pointer is not returned.
     * 
     * @credential package-manager::packagemanager_limited
     */
    PackageManagerPendingCallWatcher* cancelOperation(const QString &operation, const QString &packagename, const QString &version);

public Q_SLOTS: // METHODS from com.nokia.package_manager_install_ui (Operations with UI)
    /*!
     * @brief Installs a local package file and shows relevant dialogs
     * @param filename Full path and name of the file.
     * @return Pointer to watcher class, ownership not returned.
     * @details Sends an asyncronous D-Bus method call to com.nokia.package_manager_install_ui.
     * If the call itself fails, dbusError will be emitted by 
     * PackageManagerPendingCallWatcher. If not, installation UI will determine if the 
     * user needs to be presented with any security or dependency related questions and 
     * show the necessary dialogs before calling similar method of 
     * com.nokia.package_manager interface. After the operation has finished, if there 
     * were any errors, an error dialog will be shown on top of currently active 
     * applications.
     * 
     * Ownership of PackageManagerPendingCallWatcher pointer is not returned.
     * 
     * Required Aegis token: none.
     */
	PackageManagerPendingCallWatcher* installFileUi( const QString &filename );

    /*!
     * @brief Installs a package from remote repository and shows any relevant dialogs
     * @param packagename Package name. Case-sensitive.
     * @param version Optional version specifier. 
     * @param origin Optional repository domain identifier. 
     * @param flags Additional flags. 
     * @return Pointer to watcher class, ownership not returned.
     * @details Sends an asyncronous D-Bus method call to com.nokia.package_manager_install_ui.
     * If the call itself fails, dbusError will be emitted by
     * PackageManagerPendingCallWatcher. If not, installation UI will determine if the 
     * user needs to be presented with any security or dependency related questions or
     * refresh the cache and show the necessary dialogs before calling install or update
     * method of com.nokia.package_manager interface. If there was any errors before 
     * operationStarted signal was sent, a modal error or exception dialog will be shown 
     * to the user. After that signal, exceptions are only shown if PM operations page
     * is open or if the user later manually taps the package from launcher 
     * (applications) or from PM UI list.
     * 
     * If the operation is not accepted by the user or there is a fatal error preventing
     * the operation, operationAborted will be emitted. Otherwise operationComplete will
     * be emitted when the operation finishes. Signal operationStarted will be sent 
     * before operationComplete and any progress signals are sent if the operation 
     * actually starts (no preinstallation exceptions). PackageManagerPendingCallWatcher
     * should only be used until one of these operationX signals is sent.
     * 
     * If Version not given, the latest available version will be installed. If origin
     * is given, package will only be installed if available from this repository.
     * 
     * Flags field contains comma separated list of optional parameters. Currently two
     * flags are supported. To automatically show Package Manager operations page to 
     * display progress of requested operation, give "showoperationspage" as the string.
     * The other option is to not show the 'Preparing operation' dialog while cache 
     * update is performed, by giving value "nocacheupdatedialog".
     * 
     * Ownership of PackageManagerPendingCallWatcher pointer is not returned.
     * 
     * Required Aegis token: none.
     */
	PackageManagerPendingCallWatcher* installPackageUi( const QString &packagename, const QString& version = "", const QString &origin = "", const QString &flags = "" );

    /*!
     * @brief Uninstalls a package shows relevant dialogs
     * @param packagename Package name. Case-sensitive.
     * @return Pointer to watcher class, ownership not returned.
     * @details Sends an asyncronous D-Bus method call to com.nokia.package_manager_install_ui.
     * If the call itself fails, dbusError will be emitted by 
     * PackageManagerPendingCallWatcher. If not, installation UI will
     * determine if the user needs to be presented with any security or dependency 
     * related questions and show the necessary dialogs before calling similar method of
     * com.nokia.package_manager interface. After the operation has finished, if there 
     * were any errors, an error dialog will be shown on top of currently active 
     * applications.
     * 
     * Ownership of PackageManagerPendingCallWatcher pointer is not returned.
     * 
     * Required Aegis token: none.
     */
	PackageManagerPendingCallWatcher* uninstallUi( const QString &packagename );

    /*!
     * @brief Upgrades all packages that currently have a newer version available
     * @return Pointer to watcher class, ownership not returned.
     * @details Sends an asyncronous D-Bus method call to com.nokia.package_manager_install_ui.
     * If the call itself fails, dbusError will be emitted by 
     * PackageManagerPendingCallWatcher. If not, installation UI will
     * determine if the user needs to be presented with any security or dependency 
     * related questions and show the necessary dialogs before calling similar method of
     * com.nokia.package_manager interface. After the operation has finished, if there 
     * were any errors, an error dialog will be shown on top of currently active 
     * applications.
     * 
     * Ownership of PackageManagerPendingCallWatcher pointer is not returned.
     * 
     * Required Aegis token: none.
     */
	PackageManagerPendingCallWatcher* upgradeAllUi();

    /*!
     * @brief Restores a package that has been previously installed.
     * @param packagename Package name. Case-sensitive.
     * @param version Optional version specifier. Newest version if not specified.
     * @return Pointer to watcher class, ownership not returned.
     * @details Works exactly like installPackageUi except that this only installs
     * packages that are to be restored.
     * 
     * Ownership of PackageManagerPendingCallWatcher pointer is not returned.
     * 
     * Required Aegis token: none.
     */
	PackageManagerPendingCallWatcher* restoreUi( const QString &packagename, const QString& version = "" );

    /*!
     * @brief Shows an exception to installed package.
     * @param packagename Package name. Case-sensitive.
     * @details Shows any tracked exception for the installation. Does nothing if no exception
     * is tracked.
     * 
     * Required Aegis token: none.
     */
	void showInstallationExceptionUi( const QString &packagename );

    /*!
     * @brief Refreshes package cache
     * @param origin Optionally restricts refresh to only specified Origin/Domain
     * @param flags Optional flags. Currently only "clearignored" supported.
     * @return Pointer to watcher class, ownership not returned.
     * @details Sends an asyncronous D-Bus method call to com.nokia.package_manager_install_ui.
     * If the call itself fails, dbusError will be emitted by 
     * PackageManagerPendingCallWatcher. If Package Manager is busy the operationAborted
     * signal will be emitted with error "Busy" and a busy dialog will be shown. If 
     * neither of the mentioned errors occure, the protected refreshPackageCache method 
     * from com.nokia.package_manager interface will be called and the backend will give
     * the usual operationStarted and operationComplete signals.
     * 
     * Flags field contains comma separated list of optional parameters. Currently only
     * one flag is supported. To restore any ignored updates, give "clearignored" as the
     * string.
     * 
     * Ownership of PackageManagerPendingCallWatcher pointer is not returned.
     * 
     * Required Aegis token: none.
     */
	PackageManagerPendingCallWatcher* refreshPackageCacheUi( const QString &origin = "", const QString &flags = "" );

public Q_SLOTS: // METHODS from com.nokia.package_manager_ui (Operations that launch PM UI)
    /*!
     * @brief Starts Package Manager UI application into operations page.
     * @param operation Operation name. Case-insensitive. Same operation strings as operation signals contain.
     * @param packageidentifier Package name or filename for file based oeprations. Case-sensitive. Mandatory.
     * @param version Version specifier (not required for all operation types).
     * @details Sends a syncronous D-Bus method call to com.nokia.package_manager_ui. If call
     * succeeds, operations page will be shown with progress of the specified operation.
     * The page will close itself automatically when operation is complete.
     * 
     * Required Aegis token: none.
     */
	void showOperationsPageUi( const QString& operation, const QString &packageidentifier, const QString& version = "" );

    /*!
     * @brief Starts Package Manager UI application into updates page.
     * @details Sends a syncronous D-Bus method call to com.nokia.package_manager_ui. If call
     * succeeds, updates page will be shown.
     * 
     * Required Aegis token: none.
     */
	void showUpdatesPageUi();

    /*!
     * @brief Starts Package Manager UI application into restore page.
     * @details Sends a syncronous D-Bus method call to com.nokia.package_manager_ui. If call
     * succeeds, restore page will be shown. When invoked through a D-Bus, a cache
     * update will always be done by restore page on entry.
     * 
     * Required Aegis token: none.
     */
	void showRestorePageUi();

    /*!
     * @brief Starts Package Manager UI application into details page.
     * @param type Type of package. Case-insensitive. Possible values are: Upgrade(Update works too), Installed and Restore.
     * @param packageidentifier Package name. Case-sensitive. For file based installations,
     * @param version Version specifier.
     * @details Sends a syncronous D-Bus method call to com.nokia.package_manager_ui. If call
     * succeeds, details page will be shown for specified package. If type is Upgrade and
     * specified package is not found, updates page is shown instead. If type is invalid or
     * Installed/Restore and package is not found, nothing happens with this call.
     * 
     * Required Aegis token: none.
     */
	void showDetailsPageUi( const QString& type, const QString& packagename, const QString& version );

    /*!
     * @brief Starts Package Manager UI application into installations page.
     * @param packageidentifier Package name. Case-sensitive. Optional.
     * @details Sends a syncronous D-Bus method call to com.nokia.package_manager_ui. If call
     * succeeds, installations page will be shown. If packageidentifier is given,
     * installations page opens with given package focused.
     */
    void showInstalledPageUi( const QString& packageidentifier );

signals: // SIGNALS from com.nokia.package_manager
    /*!
     * @brief A signal that is emitted when a package operation is finished
     * @details Contains information about a single package operations. If result is an empty
     * string, the operation was successfull. If not, it will contain an error code.
     * 
     * Required Aegis token: none.
     */
    void operationComplete(const QString &operation, const QString &packagename, const QString &version, const QString &result, const bool need_reboot);

    /*!
     * @brief A signal that is emitted multiple times when a package is being downloaded
     * @details Contains amount of currently transfered bytes as well as the total amount of
     * bytes to be transferred.
     * 
     * Required Aegis token: none.
     */
    void downloadProgress(const QString &operation, const QString &packagename, const QString &version, int already, int total);

    /*!
     * @brief A signal that is emitted once for a package operation when the operation starts.
     * @details Contains operation information.
     * 
     * Required Aegis token: none.
     */
    void operationStarted(const QString &operation, const QString &packagename, const QString &version);

    /*!
     * @brief A signal that is emitted multiple times during a package operation
     * @details Contains operation information as well as progress of the operation.
     * @note When a package is downloaded before the operation, percentage is frozen to
     * zero until the download is finished.
     *
     * Required Aegis token: none.
     */
    void operationProgress(const QString &operation, const QString &packagename, const QString &version, int percentage);

    /*!
     * @brief A signal that is emitted after a cache refresh
     * @details Any applications displaying catalogues of available packages or updates should
     * refetch their data from Package Manager.
     * 
     * Required Aegis token: none.
     */
    void packageListUpdated(const bool updates_changed);

    /*!
     * @brief A signal that is emitted when operation precheck fails
     * @details If operation is aborted during precheck phase for any reason (including fatal
     * errors, user cancellation and not accepting confirmation dialogs), this signal is
     * sent so that the calling application would know that the operation has failed.
     *  
     * Parameter reason will contain additional information and currently it may contain
     * either error code sent by MW or one of these values: "Cancelled" or "Busy".
     * 
     * Required Aegis token: none.
     */
    void operationAborted(const QString &operation, const QString &packageidentifier, const QString &version, const QString &reason);

    /*!
     * @brief A signal that is emitted when package is ignored
     * @details Contains information of package. Value of type can be "Update" or "Restore".
     * 
     * Required Aegis token: none.
     */
    void packageIgnored(const QString& type, const QString& packagename, const QString& version);

private:
    PackageManagerPrivate* const d_ptr;
    Q_DISABLE_COPY(PackageManager)
    Q_DECLARE_PRIVATE(PackageManager)
};

#endif
